
#include <cyg/kernel/kapi.h>
#include <stdio.h>

#include "ezs_fb.h"
#include "ezs_counter.h"
#include "ezs_trace.h"
#include <cyg/io/framebuf.h>
#include <cyg/io/framebufs/i386_fb.h>


#define STACKSIZE CYGNUM_HAL_STACK_SIZE_MINIMUM+1024
// Thread 1
static cyg_uint8 my_stack[STACKSIZE];
static cyg_handle_t threadhndl1;
static cyg_thread   threaddata;
static cyg_alarm 	alarm1;
static cyg_handle_t alarmhnd1;

static cyg_handle_t counter;

void alarm_handler(cyg_handle_t alarm, cyg_addrword_t data){
        // 'data' is a pointer to a thread handle! (see cyg_alarm_create below)
	cyg_thread_resume(*((cyg_handle_t*) data));
}


//A little helper function.
cyg_tick_count_t ms_to_ticks(cyg_uint32 ms) {
  cyg_resolution_t resolution = cyg_clock_get_resolution(cyg_real_time_clock());
  const cyg_uint64 delay_ns = ms * 1000000;
  const cyg_tick_count_t ticks = (delay_ns * resolution.divisor)/resolution.dividend; //ticks

  return ticks;
}

// A little test thread.
void thread(cyg_addrword_t arg){

  /**
   * Example for the usage of the framebuffer.
   * Comment out for simulating the SimpleScope!
   * Reuse for displaying the pds!
   **/
  int i;
  int size = 10;
  for( i = 0; i < 256; i++){
    int col = size * (i % 16);
    int row = size * (i / 16);

    ezs_fb_fill_block(col+16*8+10, row, size, size, i);

  }
  size = 8;
  for(i=0; i < 2048 / 8; i++) {
    int row = size * (i % 16);
    int col = size * (i / 16);

    ezs_fb_print_char(i, row, col, FB_WHITE);
  }

	while(1) {
		printf("Hallo!\r\n");
		cyg_thread_suspend(cyg_thread_self());
	}
}


void cyg_user_start(void){
	// Initialize framebuffer in graphic mode
	ezs_fb_init();

	// Initialize EZS counter
	ezs_counter_init();

	// Create test thread
	cyg_thread_create(11, &thread, 0, "Abtastung1", my_stack, STACKSIZE,
			&threadhndl1, &threaddata);

	cyg_clock_to_counter(cyg_real_time_clock(), &counter);

  // Create alarm. Notice the pointer to the threadhndl1 as alarm function parameter!
	cyg_alarm_create(counter, alarm_handler, (cyg_addrword_t) &threadhndl1 , &alarmhnd1, &alarm1);
	cyg_alarm_initialize(alarmhnd1, cyg_current_time() + 1, ms_to_ticks(10));
	cyg_alarm_enable(alarmhnd1);

}

